@@ -1,5 +1,6 @@
 class Common(object):
   XIAOMI_HEADER_PREFIX = "x-xiaomi-"
+  XIAOMI_META_HEADER_PREFIX = XIAOMI_HEADER_PREFIX + "meta-"
   XIAOMI_HEADER_DATE = "x-xiaomi-date"
 
   GALAXY_ACCESS_KEY_ID = "GalaxyAccessKeyId"
@@ -17,6 +18,7 @@ class Common(object):
   CACHE_CONTROL = "cache-control"
   CONTENT_ENCODING = "content-encoding"
   CONTENT_LENGTH = "content-length"
+  META_CONTENT_LENGTH = XIAOMI_META_HEADER_PREFIX+CONTENT_LENGTH
   LAST_MODIFIED = "last-modified"
   LAST_CHECKED = "last-checked"
   UPLOAD_TIME = "upload-time"
@@ -29,7 +31,9 @@ class Common(object):
   SERVER_SIDE_ENCRYPTION = XIAOMI_HEADER_PREFIX + "server-side-encryption"
   STORAGE_CLASS = XIAOMI_HEADER_PREFIX + "storage-class"
   ONGOING_RESTORE = XIAOMI_HEADER_PREFIX + "ongoing-restore"
-  RESTORE_EXPIRY_DATE = XIAOMI_HEADER_PREFIX + "restore-expiry";
+  RESTORE_EXPIRY_DATE = XIAOMI_HEADER_PREFIX + "restore-expiry"
+  COPY_SOURCE = XIAOMI_HEADER_PREFIX + "copy-source"
+  COPY_SOURCE_RANGE = XIAOMI_HEADER_PREFIX + "copy-source-range"
 
   DEFAULT_FDS_SERVICE_BASE_URI = "http://files.fds.api.xiaomi.com/"
   DEFAULT_CDN_SERVICE_URI = "http://cdn.fds.api.xiaomi.com/"
