@@ -764,7 +764,7 @@ def _upload_batch(s_path, dst_url, concurrency, autodetect_mimetype, force):
 @click.option(
     "-a", "--autodetect_mimetype", is_flag=True, help="Whether to autodetect mimetype"
 )
-@click.option("-a", "--force", is_flag=True, help="Force downloading or uploading")
+@click.option("-f", "--force", is_flag=True, help="Force downloading or uploading")
 def cp(src, dst, autodetect_mimetype, force):
   """cp command do lots of things.
 
