@@ -40,6 +40,8 @@ from fds.model.copy_object_result import CopyObjectResult
 from fds.model.timestamp_anti_stealing_link_config import TimestampAntiStealingLinkConfig
 from fds.model.list_domain_mappings_result import ListDomainMappingsResult
 from fds.model.fds_access_log_config import AccessLogConfig
+from fds.model.fds_storage_class import FDSStorageClass, FDSRestoreResult
+from fds.model.bucket_info import BucketBean
 import os
 import sys
 from .utils import uri_to_bucket_and_object, to_json_object
@@ -157,6 +159,25 @@ class GalaxyFDSClient(object):
         response.status_code, response.content, headers)
       raise GalaxyFDSClientException(message)
 
+  def get_bucket(self, bucket_name):
+    '''
+    Get the bucket info of a specified bucket.
+    :param bucket_name: the bucket name of the specified bucket
+    :return: The bucket info
+    '''
+    uri = '%s%s' % (self._config.get_base_uri(), quote(bucket_name))
+    response = self._request.get(uri, auth=self._auth)
+    if response.status_code == requests.codes.ok:
+      bucket_bean = BucketBean(to_json_object(response.content))
+      return bucket_bean
+    else:
+      headers = ""
+      if self._config.debug:
+        headers = ' header=%s' % response.headers
+      message = 'Get bucket info failed, status=%s, reason=%s%s' % (
+        response.status_code, response.content, headers)
+      raise GalaxyFDSClientException(message)
+
   def list_buckets(self):
     '''
     List all the buckets of the current developer.
@@ -206,13 +227,16 @@ class GalaxyFDSClient(object):
     else:
       return list()
 
-  def create_bucket(self, bucket_name):
+  def create_bucket(self, bucket_name, storage_class=None):
     '''
     Create a bucket with the specified name.
     :param bucket_name: The name of the bucket to create
     '''
     uri = '%s%s' % (self._config.get_base_uri(), quote(bucket_name))
-    response = self._request.put(uri, auth=self._auth)
+    params = None
+    if storage_class:
+      params = {"storageClass": str(storage_class)}
+    response = self._request.put(uri, auth=self._auth, params=params)
     if response.status_code != requests.codes.ok:
       headers = ""
       if self._config.debug:
@@ -270,6 +294,30 @@ class GalaxyFDSClient(object):
                 (bucket_name, prefix, response.status_code, response.content, headers)
       raise GalaxyFDSClientException(message)
 
+
+  def migrate_bucket_v2(self, bucket_name, org_id, team_id):
+    '''
+    tansfer owner id of bucket to org_id and team_id, this method can not be invoke by common user
+    :param bucket_name: The name of the bucket to whom the object is put
+    :param org_id:      The target org id
+    :param team_id:   The target team id
+    '''
+    uri = '%s%s' % (self._config.get_base_uri(), quote(bucket_name))
+    params = {
+      'teamId': team_id,
+      'orgId': org_id,
+      'migrate2': ''
+    }
+    response = self._request.put(uri, auth=self._auth, params=params)
+    if response.status_code != requests.codes.ok:
+      headers = ""
+      if self._config.debug:
+        headers = ' header=%s' % response.headers
+      message = 'Failed to migrate bucket: %s to org %s with %s, ' \
+                'status=%s, reason=%s%s' % \
+                (bucket_name, org_id, team_id, response.status_code, response.content, headers)
+      raise GalaxyFDSClientException(message)
+
   def list_trash_objects(self, prefix='', delimiter=None, max_keys=None):
     '''
     Compared with list_objects, it returns a list of objects in the trash.
@@ -293,12 +341,15 @@ class GalaxyFDSClient(object):
     delimiter = previous.delimiter
     marker = previous.next_marker
     uri = "%s%s" % (self._config.get_base_uri(), quote(bucket_name))
-    response = self._request.get(uri, auth=self._auth, params={
+
+    params = {
       'prefix': previous.prefix,
       'delimiter': previous.delimiter,
       'marker': previous.next_marker,
       'maxKeys': previous.max_keys
-    })
+    }
+
+    response = self._request.get(uri, auth=self._auth, params=params)
     if response.status_code == requests.codes.ok:
       objects_list = FDSObjectListing(to_json_object(response.content))
       return objects_list
@@ -337,6 +388,9 @@ class GalaxyFDSClient(object):
     threshold_size = self._config.get_threshold_size()
     inputstream = None
 
+    if metadata == None:
+      metadata = FDSObjectMetadata()
+
     if isinstance(data, bytes):
       inputstream = BytesIO(data)
     elif isinstance(data, str):
@@ -366,21 +420,30 @@ class GalaxyFDSClient(object):
       return self.put_object_directly(bucket_name, object_name, inputstream, metadata)
     else:
       content = inputstream.read(part_size)
-      upload_token = self.init_multipart_upload(bucket_name, object_name)
+      metadata.add_header(Common.MULITPART_UPLOAD_MODE, "DIRECT")
+      upload_token = self.init_multipart_upload(bucket_name, object_name, metadata)
+      metadata.remove_header(Common.MULITPART_UPLOAD_MODE)
       upload_list = []
       part_number = 1
       while (content):
 
         upload_result = None
+        upload_success_flag = False
+        last_exception = None
         for i in range(3):
           try:
             upload_result = self.upload_part(bucket_name, object_name, upload_token.upload_id,
                                              part_number, content)
             upload_list.append(upload_result)
+            upload_success_flag = True
             break
-          except:
+          except Exception as e:
+            last_exception = e
             logging.warning("upload part %d failed, retry after %d seconds" % (part_number, 3))
             sleep(3)
+        if not upload_success_flag:
+          logging.error("upload failed, bucket: %s, object: %s, upload part: %d" % (bucket_name, object_name, part_number))
+          raise last_exception
 
         part_number = part_number + 1
         content = inputstream.read(part_size)
@@ -488,7 +551,7 @@ class GalaxyFDSClient(object):
     :return:            The FDS object
     '''
     bucket_name, object_name = uri_to_bucket_and_object(uri)
-    return self.get_object(bucket_name, object_name, position, size)
+    return self.get_object(bucket_name, object_name, position, size, True)
 
   def get_object(self, bucket_name, object_name, position=0, size=4096, stream=None,
                  version_id=None):
@@ -521,7 +584,13 @@ class GalaxyFDSClient(object):
       summary = FDSObjectSummary()
       summary.bucket_name = bucket_name
       summary.object_name = object_name
-      summary.size = int(response.headers['content-length'])
+
+      if FDSObjectMetadata.USER_DEFINED_METADATA_PREFIX + 'content-length' in response.headers:
+        summary.size = int(response.headers[FDSObjectMetadata.USER_DEFINED_METADATA_PREFIX + 'content-length'])
+      else:
+        if 'content-length' in response.headers:
+          summary.size = int(response.headers['content-length'])
+
       obj.summary = summary
       obj.metadata = self._parse_object_metadata_from_headers(response.headers)
       return obj
@@ -540,7 +609,8 @@ class GalaxyFDSClient(object):
   def download_object(self, bucket_name, object_name, data_file, offset=0, length=-1):
     fds_object = self.get_object(bucket_name=bucket_name,
                                  object_name=object_name,
-                                 position=offset)
+                                 position=offset,
+                                 stream=True)
     length_left = length
     if length_left == -1:
       length_left = IS_PY3 and sys.maxsize or sys.maxint
@@ -588,6 +658,27 @@ class GalaxyFDSClient(object):
         response.status_code, response.content, headers)
       raise GalaxyFDSClientException(message)
 
+  def get_object_metadata(self, bucket_name, object_name):
+    '''
+    Get object metadata.
+    :param bucket_name: The name of the bucket
+    :param object_name: The name of the object to check
+    :return: metadata of specified object
+    '''
+    uri = '%s%s/%s' % (self._config.get_base_uri(), quote(bucket_name), quote(object_name))
+    response = self._request.head(uri, auth=self._auth)
+    if response.status_code == requests.codes.ok:
+      return self._parse_object_metadata_from_headers(response.headers)
+    elif response.status_code == requests.codes.not_found:
+      raise GalaxyFDSClientException("Object [%s/%s] not exists" % (bucket_name, object_name))
+    else:
+      headers = ""
+      if self._config.debug:
+        headers = ' header=%s' % response.headers
+      message = 'Head object failed, status=%s, reason=%s%s' % (
+        response.status_code, response.content, headers)
+      raise GalaxyFDSClientException(message)
+
   def delete_object(self, bucket_name, object_name, **kwargs):
     '''
     Delete specified object.
@@ -826,7 +917,8 @@ class GalaxyFDSClient(object):
     '''
     uri = '%s%s/%s' % (self._config.get_base_uri(), quote(bucket_name), quote(object_name))
     acp = self._acl_to_acp(acl)
-    response = self._request.put(uri, auth=self._auth, params={'acl': 'true', 'action': "delete"},
+    params = {'acl': 'true', 'action': "delete"}
+    response = self._request.put(uri, auth=self._auth, params=params,
                                  data=json.dumps(acp, default=lambda x: x.to_string()))
     if response.status_code != requests.codes.ok:
       headers = ""
@@ -845,7 +937,8 @@ class GalaxyFDSClient(object):
     '''
     uri = '%s%s/%s' % (self._config.get_base_uri(), quote(bucket_name), quote(object_name))
     acp = self._acl_to_acp(acl)
-    response = self._request.put(uri, auth=self._auth, params={SubResource.ACL: 'true'},
+    params = {SubResource.ACL: 'true'}
+    response = self._request.put(uri, auth=self._auth, params=params,
                                  data=json.dumps(acp, default=lambda x: x.to_string()))
     if response.status_code != requests.codes.ok:
       headers = ""
@@ -855,6 +948,32 @@ class GalaxyFDSClient(object):
         response.status_code, response.content, headers)
       raise GalaxyFDSClientException(message)
 
+  def set_object_metadata(self, bucket_name, object_name, metadata):
+    """
+    Set metadata of object_name
+    :param bucket_name: bucket name
+    :param object_name: object name
+    :param metadata: a FDSObjectMetadata type object or a python dict type object
+    """
+    uri = '%s%s/%s' % (self._config.get_base_uri(), quote(bucket_name), quote(object_name))
+
+    if isinstance(metadata, dict):
+      metadataBean = {"rawMeta": metadata}
+    elif isinstance(metadata, FDSObjectMetadata):
+      metadataBean = {"rawMeta": metadata.metadata}
+    else:
+      raise GalaxyFDSClientException("the type of metadata should be FDSObjectMetadata or dict")
+    params = {"setMetaData": ''}
+    response = self._request.put(uri, auth=self._auth, params=params,
+                                 data=json.dumps(metadataBean, default=lambda x: x.to_string()))
+    if response.status_code != requests.codes.ok:
+      headers = ""
+      if self._config.debug:
+        headers = ' header=%s' % response.headers
+      message = 'Set object metadata failed, status=%s, reason=%s%s' % (
+        response.status_code, response.content, headers)
+      raise GalaxyFDSClientException(message)
+
   def get_object_acl(self, bucket_name, object_name):
     '''
     Get the ACL of a specified object.
@@ -863,7 +982,9 @@ class GalaxyFDSClient(object):
     :return: The got access control list
     '''
     uri = '%s%s/%s' % (self._config.get_base_uri(), quote(bucket_name), quote(object_name))
-    response = self._request.get(uri, auth=self._auth, params={SubResource.ACL: 'true'})
+
+    params = {SubResource.ACL: 'true'}
+    response = self._request.get(uri, auth=self._auth, params=params)
     if response.status_code == requests.codes.ok:
       acp = AccessControlPolicy(to_json_object(response.content))
       acl = self._acp_to_acl(acp)
@@ -903,8 +1024,18 @@ class GalaxyFDSClient(object):
     :param object_name: The name of the object
     :return: void
     '''
+    return self.prefetch_uri(bucket_name, object_name, "")
+
+  def prefetch_uri(self, bucket_name, object_name, cdn_uri):
+    """prefetch a uri
+
+    Args:
+      bucket_name (str): bucket name
+      object_name (str): bucketname
+      uri (str): uri string
+    """
     uri = '%s%s/%s' % (self._config.get_base_uri(), quote(bucket_name), quote(object_name))
-    response = self._request.put(uri, auth=self._auth, data="", params={"prefetch": 'true'})
+    response = self._request.put(uri, auth=self._auth, data=cdn_uri, params={"prefetch": 'true'})
     if response.status_code != requests.codes.ok:
       headers = ""
       if self._config.debug:
@@ -920,8 +1051,18 @@ class GalaxyFDSClient(object):
     :param object_name: The name of the object
     :return: void
     '''
+    return self.refresh_uri(bucket_name, object_name, "")
+
+  def refresh_uri(self, bucket_name, object_name, cdn_uri):
+    """refresh uri
+
+    Args:
+      bucket_name (str): bucket name
+      object_name (str): object name
+      uri (str): uri string
+    """
     uri = '%s%s/%s' % (self._config.get_base_uri(), quote(bucket_name), quote(object_name))
-    response = self._request.put(uri, auth=self._auth, data="", params={"refresh": 'true'})
+    response = self._request.put(uri, auth=self._auth, data=cdn_uri, params={"refresh": 'true'})
     if response.status_code != requests.codes.ok:
       headers = ""
       if self._config.debug:
@@ -930,29 +1071,64 @@ class GalaxyFDSClient(object):
         response.status_code, response.content, headers)
       raise GalaxyFDSClientException(message)
 
-  def set_public(self, bucket_name, object_name):
+  def set_public(self, bucket_name, object_name=None):
+    """
+    Make resource public, if object_name is not being set, this method will set the bucket_name to public resource,
+    if object_name is set, this method will set object_name to public resource
+    :param bucket_name:
+    :param object_name:
+    :return:
+    """
+    if object_name is None:
+      self._set_bucket_public(bucket_name)
+    else:
+      self._set_object_public(bucket_name, object_name)
+
+  def _set_object_public(self, bucket_name, object_name):
     acl = AccessControlList()
     grant = Grant(Grantee(UserGroups.ALL_USERS), Permission.READ)
     grant.type = GrantType.GROUP
     acl.add_grant(grant)
     self.set_object_acl(bucket_name, object_name, acl)
 
-  def set_public(self, bucket_name):
+  def _set_bucket_public(self, bucket_name):
     acl = AccessControlList()
     grant = Grant(Grantee(UserGroups.ALL_USERS), Permission.READ_OBJECTS)
     grant.type = GrantType.GROUP
     acl.add_grant(grant)
     self.set_bucket_acl(bucket_name, acl)
 
-  def init_multipart_upload(self, bucket_name, object_name):
+  def set_private(self, bucket_name, object_name=None):
+    if object_name is None:
+      self._set_bucket_private(bucket_name)
+    else:
+      self._set_object_private(bucket_name, object_name)
+
+  def _set_bucket_private(self, bucket_name):
+    acl = AccessControlList()
+    grant = Grant(Grantee(UserGroups.ALL_USERS), Permission.READ_OBJECTS)
+    grant.type = GrantType.GROUP
+    acl.add_grant(grant)
+    self.delete_bucket_acl(bucket_name, acl)
+
+  def _set_object_private(self, bucket_name, object_name):
+    acl = AccessControlList()
+    grant = Grant(Grantee(UserGroups.ALL_USERS), Permission.READ)
+    grant.type = GrantType.GROUP
+    acl.add_grant(grant)
+    self.delete_object_acl(bucket_name, object_name, acl)
+
+  def init_multipart_upload(self, bucket_name, object_name, metadata=None):
     '''
     Init a multipart upload session
     :param bucket_name:
     :param object_name:
     :return:
     '''
+    if not metadata:
+      metadata = FDSObjectMetadata()
     uri = '%s%s/%s' % (self._config.get_base_uri(), quote(bucket_name), quote(object_name))
-    response = self._request.put(uri, auth=self._auth, data="", params={"uploads": 'true'})
+    response = self._request.put(uri, auth=self._auth, data="", params={"uploads": 'true'}, headers=metadata.metadata)
     if response.status_code == requests.codes.ok:
       result = InitMultipartUploadResult(to_json_object(response.content))
       return result
@@ -964,6 +1140,19 @@ class GalaxyFDSClient(object):
         response.status_code, response.content, headers)
       raise GalaxyFDSClientException(message)
 
+  def init_multipart_upload_copy(self, bucket_name, object_name, metadata=None):
+    '''
+    Init a multipart upload copy session
+    :param bucket_name:
+    :param object_name:
+    :param metadata:
+    :return:
+    '''
+    if not metadata:
+      metadata = FDSObjectMetadata()
+    metadata.add_header(Common.MULITPART_UPLOAD_MODE, "MULTI_BLOB")
+    return self.init_multipart_upload(bucket_name, object_name, metadata)
+
   def upload_part(self, bucket_name, object_name, upload_id, part_number, data):
     '''
     Upload a multipart upload part
@@ -990,6 +1179,43 @@ class GalaxyFDSClient(object):
         response.status_code, response.content, headers)
       raise GalaxyFDSClientException(message)
 
+  def upload_part_copy(self, bucket_name, object_name, upload_id, part_number, source_bucket_name,
+                       source_object_name, begin_range=None, end_range=None):
+    '''
+    Copy a multipart upload part
+    :param bucket_name:
+    :param object_name:
+    :param upload_id:
+    :param part_number:
+    :param source_bucket_name:
+    :param source_object_name:
+    :param begin_range:
+    :param end_range:
+    :return:
+    '''
+    uri = '%s%s/%s' % (self._config.get_base_uri(), quote(bucket_name), quote(object_name))
+
+    metadata = FDSObjectMetadata()
+    metadata.add_header(Common.COPY_SOURCE,
+                        '/%s/%s' % (quote(source_bucket_name), quote(source_object_name)))
+    if begin_range is not None and end_range is not None:
+      metadata.add_header(Common.COPY_SOURCE_RANGE, "bytes=%d-%d" % (begin_range, end_range))
+
+    response = self._request.put(uri, auth=self._auth, params={
+      "uploadId": upload_id,
+      "partNumber": str(part_number)
+    }, headers=metadata.metadata)
+    if response.status_code == requests.codes.ok:
+      result = UploadPartResult(to_json_object(response.content))
+      return result
+    else:
+      headers = ""
+      if self._config.debug:
+        headers = ' headers=%s' % response.headers
+      message = 'Upload part failed, status=%s, reason=%s%s' % (
+        response.status_code, response.content, headers)
+      raise GalaxyFDSClientException(message)
+
   def complete_multipart_upload(self, bucket_name, object_name, upload_id,
                                 metadata, upload_part_result_list):
     '''
@@ -1368,49 +1594,6 @@ class GalaxyFDSClient(object):
         response.status_code, response.content, headers)
       raise GalaxyFDSClientException(message)
 
-  def _set_bucket_default_webp_quality_(self, bucket_name, quality):
-    '''
-    Enable bucket auto convert webp.
-    :param bucket_name:     The name of the bucket
-    :param quality:         Default webp quality, -1 means disable auto convert webp
-    '''
-    uri = '%s%s' % (self._config.get_base_uri(), quote(bucket_name))
-    response = self._request.put(uri, auth=self._auth, params={
-      "webpQuality": str(quality)
-    })
-    if response.status_code != requests.codes.ok:
-      headers = ""
-      if self._config.debug:
-        headers = ' header=%s' % response.headers
-      message = 'Enable bucket auto convert webp failed, status=%s, reason=%s%s' % (
-        response.status_code, response.content, headers)
-      raise GalaxyFDSClientException(message)
-
-  def _get_bucket_default_webp_quality_(self, bucket_name):
-    '''
-    Check the existence of a specified object.
-    :param bucket_name: The name of the bucket
-    :return: True if the object exists, otherwise, False
-    '''
-    uri = '%s%s' % (self._config.get_base_uri(), quote(bucket_name))
-    response = self._request.get(uri, auth=self._auth, params={
-      "webpQuality": "true"
-    })
-    if response.status_code == requests.codes.ok:
-      content = response.content
-      if IS_PY3:
-        content = content.decode("UTF-8")
-      return int(content)
-    elif response.status_code == requests.codes.not_found:
-      return False
-    else:
-      headers = ""
-      if self._config.debug:
-        headers = ' header=%s' % response.headers
-      message = 'Get bucket default webp quality failed, status=%s, reason=%s%s' % (
-        response.status_code, response.content, headers)
-      raise GalaxyFDSClientException(message)
-
   def _get_webp_(self, bucket_name, object_name, stream=None, quality=None):
     '''
     Get a specified object from a bucket.
@@ -1446,76 +1629,6 @@ class GalaxyFDSClient(object):
         response.status_code, response.content, headers)
       raise GalaxyFDSClientException(message)
 
-  def _set_bucket_default_gif_extract_type_(self, bucket_name, mime_type):
-    '''
-    Enable bucket auto gif extract.
-    :param bucket_name:     The name of the bucket
-    :param mime_type:         Default gif extract type, unknown means disable auto gif extract
-    '''
-    uri = '%s%s' % (self._config.get_base_uri(), quote(bucket_name))
-    response = self._request.put(uri, auth=self._auth, params={"gifType": mime_type})
-    if response.status_code != requests.codes.ok:
-      headers = ""
-      if self._config.debug:
-        headers = ' header=%s' % response.headers
-      message = 'Enable bucket auto gif extract failed,status=%s, reason=%s%s' % (
-        response.status_code, response.content, headers)
-      raise GalaxyFDSClientException(message)
-
-  def _get_extracted_gif_(self, bucket_name, object_name, type, stream=None):
-    '''
-    Get gif extracted frame of a specified
-    :param bucket_name: The name of the bucket from whom to get the object
-    :param object_name: The name of the object to get
-    :param stream:      Set True to enable streaming, otherwise, whole object content is read to memory
-    :param type: The FDS object
-    :return:
-    '''
-    uri = '%s%s/%s' % (self._config.get_download_base_uri(), quote(bucket_name), quote(object_name))
-    params = {"gf": type}
-    response = self._request.get(uri, auth=self._auth, stream=stream, params=params)
-
-    if response.status_code == requests.codes.ok or response.status_code == requests.codes.partial:
-      obj = FDSObject()
-      obj.stream = response.iter_content()
-      summary = FDSObjectSummary()
-      summary.bucket_name = bucket_name
-      summary.object_name = object_name
-      summary.size = int(response.headers['content-length'])
-      obj.summary = summary
-      obj.metadata = self._parse_object_metadata_from_headers(response.headers)
-      return obj
-    else:
-      headers = ""
-      if self._config.debug:
-        headers = ' header=%s' % response.headers
-      message = 'Get extracted gif failed, status=%s, reason=%s%s' % (
-      response.status_code, response.content, headers)
-      raise GalaxyFDSClientException(message)
-
-  def _get_bucket_default_gif_extract_type_(self, bucket_name):
-    '''
-    Check the existence of a specified bucket.
-    :param bucket_name: The name of the bucket
-    :return: Type if the bucket can auto gif extract, otherwise, unknown
-    '''
-    uri = '%s%s' % (self._config.get_base_uri(), quote(bucket_name))
-    response = self._request.get(uri, auth=self._auth, params={"gifType": "true"})
-    if response.status_code == requests.codes.ok:
-      content = response.content
-      if IS_PY3:
-        content = content.decode("UTF-8")
-      return content
-    elif response.status_code == requests.codes.not_found:
-      return False
-    else:
-      headers = ""
-      if self._config.debug:
-        headers = ' header=%s' % response.headers
-      message = 'Get bucket default gif extract type failed, status=%s, reason=%s%s' % (
-        response.status_code, response.content, headers)
-      raise GalaxyFDSClientException(message)
-
   def generate_anti_stealing_uri(self, bucket_name, object_name, expiration, key):
     '''
     Generate anti stealing URI
@@ -1591,24 +1704,30 @@ class GalaxyFDSClient(object):
         resp.status_code, resp.content, headers)
       raise GalaxyFDSClientException(message)
 
-  def _restore_archived_object_(self, src_bucket_name, src_object_name, dst_bucket_name, dst_object_name):
-      '''
-      Restore archived object as standard.
-      If the object is ongoing restore, or not archived, raise exception
-      :param bucket_name:     The name of the bucket
-      :param object_name:     The name of the object
-      '''
-      uri = '%s%s/%s' % (self._config.get_base_uri(), quote(src_bucket_name), quote(src_object_name))
-      response = self._request.put(uri, auth=self._auth,
-        params={"restoreArchive": "true"},
-        data=("%s/%s" % (dst_bucket_name, dst_object_name)))
-      if response.status_code != requests.codes.ok:
-          headers = ""
-          if self._config.debug:
-              headers = ' header=%s' % response.headers
-          message = 'Restore archived object failed, status=%s, reason=%s%s' % (
-              response.status_code, response.content, headers)
-          raise GalaxyFDSClientException(message)
+  def _restore_archived_object_(self, src_bucket_name, src_object_name):
+    '''
+    Restore archived object
+    If object is restored, extend expiration time of the copy
+    :param bucket_name:     The name of the bucket
+    :param object_name:     The name of the object
+    '''
+    uri = '%s%s/%s' % (self._config.get_base_uri(), quote(src_bucket_name), quote(src_object_name))
+    response = self._request.put(uri, auth=self._auth,
+                                 params={"restoreArchive": "true"})
+
+    if response.status_code == 200:
+      return FDSRestoreResult.Restored
+    elif response.status_code == 202:
+      return FDSRestoreResult.Restore
+    elif response.status_code == 409:
+      return FDSRestoreResult.Restoring
+    else:
+      headers = ""
+      if self._config.debug:
+        headers = ' header=%s' % response.headers
+      message = 'Restore archived object failed, status=%s, reason=%s%s' % (
+        response.status_code, response.content, headers)
+      raise GalaxyFDSClientException(message)
 
   def set_object_outside_access(self, bucket_name, object_name, value):
     if not isinstance(value, bool):
@@ -1634,4 +1753,4 @@ class GalaxyFDSClient(object):
         headers = ' header=%s' % response.headers
       message = 'Set bucket outsideAccess failed, status=%s, reason=%s%s' % (
         response.status_code, response.content, headers)
-      raise GalaxyFDSClientException(message)
\ No newline at end of file
+      raise GalaxyFDSClientException(message)
