@@ -84,6 +84,8 @@ class FDSLifecycleRule(dict):
         self._actions[name] = FDSExpiration(action)
       elif name == 'nonCurrentVersionExpiration':
         self._actions[name] = FDSNonCurrentVersionExpiration(action)
+      elif name == 'lifeCycleStorageClass':
+        self._actions[name] = FDSLifecycleStorageClass(action)
       else:
         raise GalaxyFDSClientException("invalid action type: " + name)
     self['actions'] = self._actions
@@ -157,6 +159,24 @@ class FDSAbortIncompleteMultipartUpload(dict):
   def days(self, days):
     self['days'] = days
 
+class FDSLifecycleStorageClass(dict):
+  name = 'lifeCycleStorageClass'
+
+  @property
+  def days(self):
+    return self.get('days', 0)
+
+  @days.setter
+  def days(self, days):
+    self['days'] = days
+
+  @property
+  def storage_class(self):
+    return self.get('storageClass', "")
+
+  @storage_class.setter
+  def storage_class(self, storage_class):
+    self['storageClass'] = storage_class
 
 if __name__ == '__main__':
   lifecycle_config = FDSLifecycleConfig()
@@ -201,6 +221,10 @@ if __name__ == '__main__':
          },
          "abortIncompleteMultipartUpload": {
            "days":7
+         },
+         "lifeCycleStorageClass": {
+           "days":2,
+           "storageClass":"ARCHIVE"
          }
        }
      }
