@@ -26,7 +26,9 @@ class FDSObjectMetadata(object):
     Common.SERVER_SIDE_ENCRYPTION,
     Common.STORAGE_CLASS,
     Common.RESTORE_EXPIRY_DATE,
-    Common.ONGOING_RESTORE
+    Common.ONGOING_RESTORE,
+    Common.COPY_SOURCE,
+    Common.COPY_SOURCE_RANGE
   ]
 
   def __init__(self):
